@@ -6,13 +6,15 @@ from __future__ import absolute_import
 
 import abc
 import logging
+import operator
 
 import entrypoints
 
-from . import errors, util
+from . import credentials, errors, util
 from .util import properties
 from .py27compat import add_metaclass, filter
 
+__metaclass__ = type
 
 log = logging.getLogger(__name__)
 
@@ -32,7 +34,7 @@ class KeyringBackendMeta(abc.ABCMeta):
 
 
 @add_metaclass(KeyringBackendMeta)
-class KeyringBackend(object):
+class KeyringBackend:
     """The abstract base class of the keyring, every backend must implement
     this interface.
     """
@@ -61,6 +63,13 @@ class KeyringBackend(object):
             cls.priority
         return not bool(exc)
 
+    @classmethod
+    def get_viable_backends(cls):
+        """
+        Return all subclasses deemed viable.
+        """
+        return filter(operator.attrgetter('viable'), cls._classes)
+
     @properties.ClassProperty
     @classmethod
     def name(cls):
@@ -73,6 +82,12 @@ class KeyringBackend(object):
         mod_name = mod_name.replace('_', ' ')
         return ' '.join([mod_name, cls.__name__])
 
+    def __str__(self):
+        keyring_class = type(self)
+        return ("%s.%s (priority: %g)" % (keyring_class.__module__,
+                                          keyring_class.__name__,
+                                          keyring_class.priority))
+
     @abc.abstractmethod
     def get_password(self, service, username):
         """Get password of the username for the service
@@ -81,7 +96,10 @@ class KeyringBackend(object):
 
     @abc.abstractmethod
     def set_password(self, service, username, password):
-        """Set password for the username of the service
+        """Set password for the username of the service.
+
+        If the backend cannot store passwords, raise
+        NotImplementedError.
         """
         raise errors.PasswordSetError("reason")
 
@@ -90,11 +108,35 @@ class KeyringBackend(object):
     # @abc.abstractmethod
     def delete_password(self, service, username):
         """Delete the password for the username of the service.
+
+        If the backend cannot store passwords, raise
+        NotImplementedError.
         """
         raise errors.PasswordDeleteError("reason")
 
+    # for backward-compatibility, don't require a backend to implement
+    #  get_credential
+    # @abc.abstractmethod
+    def get_credential(self, service, username):
+        """Gets the username and password for the service.
+        Returns a Credential instance.
 
-class Crypter(object):
+        The *username* argument is optional and may be omitted by
+        the caller or ignored by the backend. Callers must use the
+        returned username.
+        """
+        # The default implementation requires a username here.
+        if username is not None:
+            password = self.get_password(service, username)
+            if password is not None:
+                return credentials.SimpleCredential(
+                    username,
+                    password,
+                )
+        return None
+
+
+class Crypter:
     """Base class providing encryption and decryption
     """
 
@@ -159,15 +201,6 @@ def get_all_keyring():
     parameters.
     """
     _load_plugins()
-
-    def is_class_viable(keyring_cls):
-        try:
-            keyring_cls.priority
-        except RuntimeError:
-            return False
-        return True
-
-    all_classes = KeyringBackend._classes
-    viable_classes = filter(is_class_viable, all_classes)
-    return list(util.suppress_exceptions(viable_classes,
-                                         exceptions=TypeError))
+    viable_classes = KeyringBackend.get_viable_backends()
+    rings = util.suppress_exceptions(viable_classes, exceptions=TypeError)
+    return list(rings)
